class NotFoundException(Exception):
    pass

class Registry(object):
    class __metaclass__(type):
        def __new__(cls, name, bases, attrs):
            data = attrs.get('data', {})
            attrs['data'] = data.copy()
            return type.__new__(cls, name, bases, attrs)

    data = {}

    @classmethod
    def resolve(cls, name, default=None):
        name = name.lower()
        if name in cls.data:
            return cls.data[name]
        elif default is not None:
            return default
        else:
            Exc = getattr(cls, 'NotFound', NotFoundException)
            raise Exc('Key %r not found in %r' % (name, cls))

    @classmethod
    def register(cls, val, name):
        # XXX: I'm not sure about that. Don't looks healthy
        for x in getattr(val, 'alter_names', []):
            cls.data[x] = val

        cls.data[name] = val

    @classmethod
    def list(cls):
        return cls.data.keys()



# -*- coding: utf-8 -*-
#

class BaseDatasource(object):
    """ Data source interface
    """

    def __init__(self, conn):
        self.conn = conn

    def put(self):
        raise NotImplemented

    def get(self):
        raise NotImplemented

    def get_by_id(self):
        raise NotImplemented

    def delete(self):
        raise NotImplemented

    def put_multi(self):
        raise NotImplemented

    def get_multi(self):
        raise NotImplemented

    def del_multi(self):
        raise NotImplemented

    def query(self):
        raise NotImplemented

    def count(self):
        raise NotImplemented

    def commit(self):
        raise NotImplemented

    def rollback(self):
        raise NotImplemented



# -*- coding: utf-8 -*-

import datetime, re

import controllers
import widgets
from registry import Registry
from formgear.exceptions import InvalidValue

class NotFoundFieldException(Exception):
    pass


class FieldsRegistry(Registry):
    """ Registry needed to resolve fields from YAML file """
    fields = {}
    NotFound = NotFoundFieldException


class MetaField(type):
    """
    Class for all widgets
    """
    def __new__(cls, name, bases, attrs):
        meta = attrs.pop('Meta', None)
        abstract = getattr(meta, 'abstract', False)
        registername = attrs.pop('name', name.lower())
        newbornclass = super(MetaField, cls).__new__(cls, name, bases, attrs)

        if not abstract:
            #print("Register widget:", registername)
            FieldsRegistry.register(newbornclass, registername)

        return newbornclass


def init_partial(real):
    def init_partial(self, *a, **kw):
        self._partial = a, kw
        return real(self, *a, **kw)

    return init_partial


class BaseField(object):
    """ BaseField is very similar to MongoEngine fields.
    """

    __metaclass__ = MetaField
    # Fields may have _types inserted into indexes by default
    _index_with_types = True
    _geo_index = False
    widget = widgets.StringWidget

    @init_partial
    def __init__(self, db_field=None, required=False, default=None,
                 unique=False, unique_with=None, primary_key=False,
                 validators=[], widget=None, **kw):
        """
        Params:

          * db_field - set explicit field name in database
          * required - quick alias for validator.Required
          * title - field name
          * default=None - default value
          * unique=False - mongodb unique field index
          * unique_with=None - mongodb unique_with field index
          * primary_key=False - mongodb field index for primary_key
          * validators=None - set of validators from controllers collection (or your own)
          * description=None - field help text description
          * widget - widget to represent field value

        """
        self.db_field = db_field if not primary_key else '_id'

        self.required = required or primary_key
        self.unique = bool(unique or unique_with)
        self.unique_with = unique_with
        self.default = default

        self.validators = controllers.lookup(validators)
        if controllers.Required in self.validators:
            self.required = True
        if required:
            self.required = True
            if controllers.Required not in self.validators:
                self.validators.append(controllers.Required)

        if widget:
            self.widget = widget

        self.__dict__.update(kw)
        self.value = None

    def validate(self):
        """Perform validation on a value.
        """
        ret = True
        self.errors = []
        for validator in self.validators:
            if isinstance(validator, type):
                validator = validator()

            try:
                validator(self, self.value)
            except InvalidValue, e:
                ret = False
                self.errors.append(e.args[1])
        return ret

    def reinstance(self):
        a, kw = self._partial
        return self.__class__(*a, **kw)

    def translate(self, msgid):
        """ Use the translator passed to the renderer of this field to
        translate the msgid into a term.  If the renderer does not have a
        translator, this method will return the msgid."""
        translate = getattr(self.renderer, 'translate', None)
        if translate is not None:
            return translate(msgid)
        return msgid

    def __repr__(self):
        return '<%s.%s object at %d>' % (
            self.__module__,
            self.__class__.__name__,
            id(self)
            )

    def __unicode__(self):
        return unicode(self.value)

    __str__ = __unicode__

    def __call__(self, state="edit", **kwargs):
        return self.widget.render(self, state, **kwargs)

    def clear(self):
        self._value = None

    def set_value(self, val):
        self._value = val

    def get_value(self):
        val = self._value or self.default

        if val:
            short, _val = self.shortcut(val)
            if short:
                val = _val

        return val

    value = property(get_value, set_value)

    def shortcut(self, value):
        return False, None

    @property
    def to_mongo(self):
        if hasattr(self, '__mongo__'):
            val = self.__mongo__(self.value)
        else:
            val = self.value

        return val


class StringField(BaseField):
    """ Simple string
    """

    alter_names = ('string',)

    def __init__(self, regex=None, max_length=None, min_length=None, **kwargs):
        self.regex = re.compile(regex) if regex else None
        self.max_length = max_length
        self.min_length = min_length
        super(StringField, self).__init__(**kwargs)


class TextField(BaseField):
    """ Plain text field.
    XXX: Add text processors, may be including typography.
    """
    alter_names = ('text',)
    widget = widgets.TextWidget

    def __init__(self, max_length=None, min_length=None, **kwargs):
        self.max_length = max_length
        self.min_length = min_length
        super(TextField, self).__init__(**kwargs)


class DateField(BaseField):
    alter_names = ('date', )

    format='%Y/%m/%d'
    # self.data = datetime.datetime.strptime(date_str, self.format).date()
    def __init__(self, **kwargs):
        """
        """
        pass


class DateTimeField(BaseField):

    alter_names = ('datetime',)
    type = 'date'

    def shortcut(self, value):
        if value == 'now':
            return True, datetime.datetime.now()
        elif value == 'today':
            return True, datetime.datetime.today()

        return False, None

    def validate(self):
        if isinstance(self.value, datetime.datetime):
            return True

        from dateutil import parser
        try:
            self.value = parser.parse(self.value or '')
        except ValueError:
            return

        return True


class TimeField(BaseField):
    alter_names = ('time', )


class IntegerField(BaseField):
    alter_names = ('int', 'integer', )


class BooleanField(BaseField):
    alter_names = ('bool', 'boolean', )

    def shortcut(self, value):
        return True, bool(value)

class EmailField(BaseField):
    alter_names = ('email', )

    def validate(self):
        ret = '@' in self.value
        if not ret:
            self.errors = [
                    u"Wrong email format",
            ]
        return ret


class FloatField(BaseField):
    alter_names = ('float', )


class URLField(BaseField):
    alter_names = ('url', 'link', )

    #XXX: Y NO ccTLD like xn--shitshitshit domains?
    URL_REGEX = re.compile(
            r'^https?://'
            r'(?:(?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\.)+[A-Z]{2,6}\.?|'
            r'localhost|'
            r'\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})'
            r'(?::\d+)?'
            r'(?:/?|[/?]\S+)$', re.IGNORECASE
        )

    def __init__(self, verify_exists=False):
        self.verify_exists = verify_exists
        super(URLField, self).__init__(**kwargs)


class FileField(BaseField):
    alter_names = ('file', 'blob', )


class ImageField(BaseField):
    alter_names = ('img', 'image', )


class GeoPointField(BaseField):
    alter_names = ('geo', 'geopoint', )


class TimerangeField(BaseField):
    alter_name = ('timerange')
    widget = widgets.TimerangeWidget


class CheckboxField(BaseField):
    alter_names = ('checkbox',)
    choices = {}
    widget = widgets.CheckboxWidget

    def clear(self,):
        self._value = []

    def set_value(self, val):

        oldval = getattr(self, '_value', None)
        if not isinstance(oldval, list):
            self._value = [] if oldval is None else [oldval]

        if isinstance(val, (list, tuple)):
            self._value = val
        else:
            self._value.append(val)

    value = property(BaseField.get_value, set_value)


class StructField(BaseField):
    alter_names = ('struct',)


class PathField(StringField):
    alter_names = ('pathfield',)

    def set_value(self, val):
        if val:
            if not val.startswith("/"):
                val = "/%s" % (val,)
            self._value = val

    value = property(BaseField.get_value, set_value)



from pymongo import Connection

connection = Connection()
db = connection['comfortly']

def save(collection, data, _id=None):
    col = db[collection]
    _id = _id or data.get('_id')
    if not (_id is None):
        col.update({"_id": _id}, data, upsert=True, safe=True)
        return _id
    else:
        return col.insert(data)

def find(collection, **kw):
    col = db[collection]

    return col.find(kw)

def remove(collection, _filter):
	col = db[collection]
	#print collection, _filter
	col.remove(_filter)



"""
List of exceptions that may be happen in our new modest framework.
"""

class NotFoundModelException(Exception):
    pass


class ParsingExcpetion(Exception):
    pass


class YamlAttributeNotFoundException(Exception):
    pass


class YamlEntryNotFoundInListException(Exception):
    pass

class NotFoundValidatorException(Exception):
    pass

class InvalidValue(Exception):
    """
    Base form validation exception

    Based on https://github.com/Pylons/colander/blob/master/colander/__init__.py

    """



# -*- coding: utf-8 -*-
#
from __future__ import print_function
import os
import yaml
import mongo

from formgear.fields import FieldsRegistry
from formgear.widgets import WidgetRegistry
from registry import Registry
from formgear.exceptions import *
from formgear.utils import yamls_files, file_resolve

from jinja2 import Environment, PackageLoader

__author__ = 'xen'

yamlsfiles = yamls_files()


class ModelRegistry(Registry):
    NotFound = NotFoundModelException

def model_wrap(model, **wrap_kw):
    def __init__(self, *a, **kw):
        kw.update(wrap_kw)
        model.__init__(self, *a, **kw)

    attrs = {"__init__": __init__, "__fake__": True}
    return MetaModel(model.__name__, (model,), attrs)

class FormWrap(object):
    def __init__(self, forms, model):
        self.model = model

        self.forms = forms

    def get(self, name, silent=None):
        """ Return form fields as dict for a class:

            >>> model.form(name='default')
            [('email', <formgear.fields.StringField object at 4529231184>), ...)]
            >>> model.form(name='search')
            KeyError: "Form 'search' not found for model <class 'admin.auth.models.User'>"

        There is always 'default' form which contaions all model field.

        If `silent` param provided then wrong form name don't caught exception

        """
        name = name or 'default'
        for form in self.forms:
            if form['name'] == name:
                #print(name, ': ', form)
                return form
        if silent:
            return None
        else:
            raise KeyError("Form %r not found for model %r" % (name, self.model))

    def field(self, name):
        return self.model._fields_dict[name]

    def __call__(self, name=None, fields=[], **kw):
        if not fields:
            form = self.get(name or self.model.subform, **kw)
            if form:
                fields = form['fields']

        ret = [
                (name, self.field(name))
                for name in fields
        ]
        return ret


    def __getattribute__(self, name):
        try:
            return object.__getattribute__(self, name)
        except AttributeError:
            if self.get(name) is None:
                raise

        if isinstance(self.model, type):
            model = model_wrap(self.model, subform=name)
            model.subform = name
            return model

        obj = self.model.__class__(subform=name)
        obj._fields = self.model._fields
        return obj


class MetaModel(type):
    """
    Base model metaclass
    """
    def __new__(cls, name, bases, attrs):
        meta = attrs.pop('Meta', None)
        abstract = getattr(meta, 'abstract', False)
        fake = attrs.pop('__fake__', None)
        registername = attrs.pop('name', name.lower())

        cfg = {}
        # we have search __yaml__ attribute only, when we
        # have initialize a subclass of formgear.models.Model
        if not abstract:

            # try to find out by __yaml__ or by class name
            # __yaml__ = "order" or class Order(Models):
            ypath = attrs.get('__yaml__') or name.lower()

            ypath = yamlsfiles.get(ypath, ypath)
            if not os.access(ypath, 0):
                raise YamlEntryNotFoundInListException

            cfg = yaml.safe_load(open(ypath))

            attrs["__yaml__"] = ypath

        # this block should make model registry list looks better
        _descr = ''
        if cfg.get('description', None):
            _descr = cfg.get('description')
        if attrs.get('__doc__', None) and len(attrs.get('__doc__')):
            _descr = attrs.get('__doc__')
        else:
            attrs['__doc__'] = _descr
        _title = registername
        if cfg.get('title', None):
            _title = cfg.get('title')

        cfg['_name'] = registername
        cfg['_descr'] = _descr
        cfg['_title'] = _title

        fields = []
        key_fields = attrs.get('__key__', [])
        if not isinstance(key_fields, (list,tuple)):
            key_fields = []

        for field in cfg.pop('fields', []):
            if 'name' not in field:
                raise ParsingException("Oops, we found nameless field!")

            field_typ = field.pop('type', 'string')
            field_class = FieldsRegistry.resolve(field_typ)

            # extracts widget information
            widget = field.pop('widget', field_class.widget)

            if isinstance(widget, dict):
                widget_typ = widget.pop('type', field_class.widget)
                widget_kw = widget
            else:
                widget_typ = widget
                widget_kw = {}

            if isinstance(widget_typ, basestring):
                widgt_class = WidgetRegistry.resolve(widget_typ)
            else:
                widgt_class = widget_typ

            wdgt = widgt_class(**widget_kw)

            file_resolve(field, "choices", ypath)

            if field['name'] in key_fields:
                field['required'] = True

            new_field = field_class(widget = wdgt, **field)

            # actual work with fields
            # XXX: here is missed part with validators
            fields.append((field.pop('name'), new_field))

        forms = []
        forms.extend(cfg.pop('forms', []))
        forms.append({
            "name": "default",
            "fields": [
                fname
                for fname, _field in fields
                if hasattr(_field, 'title')
            ],
        })

        cfg.update(attrs)
        newbornclass = super(MetaModel, cls).__new__(cls, name, bases, cfg)

        for fname, ffunc in fields:
            setattr(newbornclass, fname, ffunc)

        newbornclass._fields = fields
        newbornclass._fields_dict = dict(fields)


        newbornclass.form = FormWrap(forms, newbornclass)

        if not abstract and not fake:
            ModelRegistry.register(newbornclass, registername)

        return newbornclass

class Model(object):
    __metaclass__ = MetaModel
    subform = None

    class Meta:
        abstract = True

    def __init__(self, data=None, subform=None, _id=None, _raw=False, **kw):
        assert data is None or not kw, 'Pass data in one way'
        if data:
            kw = data
        if _id:
            self._id = _id

        self.subform = subform
        form = self.form.get(subform)
        self.render_form = self._render_form

        subform_fields = []
        all_fields = []
        for name, _field in self._fields:
            field = _field.reinstance()
            all_fields.append((name,field))

            if name in form['fields']:
                subform_fields.append((name, field))

        self._fields = subform_fields
        self._fields_dict = dict(subform_fields)
        self._all_fields_dict = dict(all_fields)
        self.form = FormWrap(self.form.forms, self)

        self.update(kw, raw=_raw)
        if _id:
            self.lock_id()

    def update(self, data=None, raw=False, **kw):
        assert data is None or not kw, 'Pass data in one way'
        kw = data or kw
        if callable(getattr(kw, 'items', None)):
            kw = kw.items()

        for name, val in kw:

            field = self._field(name, raw=raw)
            if not field:
                continue

            if getattr(field, 'locked', False):
                continue

            field.value = val

    def items(self):
        for name, field in self._fields:
            yield name, field.value

    def __iter__(self):
        return iter(self.form(self.subform))

    def _field(self, name, raw=False):
        if raw:
            return self._all_fields_dict.get(name)
        return self._fields_dict.get(name)

    # getattr gives access to all loaded fields
    def __getattribute__(self, name):
        try:
            fields = object.__getattribute__(self,'_all_fields_dict')
            if name in fields:
                return fields.get(name).value

        except AttributeError:
            pass

        return object.__getattribute__(self, name)

    # setattr gives acces only to fields available in this subform
    def __setattr__(self, name, value):
        try:
            fields = object.__getattribute__(self,'_fields_dict')
            if name in fields:
                fields[name].value = value
                return
            elif name in object.__getattribute__(self,'_all_fields_dict'):
                raise TypeError(
                    "Refused to update field %s missing in subform %s" %
                    (name, self.subform)
                )

        except AttributeError:
            pass

        return object.__setattr__(self, name, value)

    def validate(self):
        for name, field in self._fields:
            if not hasattr(field, 'validate'):
                continue

            valid = field.validate()
            if not valid:
                return

        return True

    def to_mongo(self):

        doc = dict([
            (name, field.to_mongo)
            for name,field in self._all_fields_dict.items()
        ])

        if '_id' in doc:
            pass
        elif hasattr(self, 'key'):
            _id = self.key()
            if not (_id is None):
                doc['_id'] = _id
        elif hasattr(self, '_id'):
           doc['_id'] = self._id

        return doc

    def lock_id(self):
        if not hasattr(self.__class__, '__key__'):
            return

        if not isinstance(self.__key__, (list, tuple)):
            return

        for name in self.__key__:
            field = self._field(name)
            if not field:
                continue

            field.locked = True

    @classmethod
    def __key_type(cls, value):
        import bson
        if not hasattr(cls, '__key__'):
            return bson.objectid.ObjectId(value)

        return value

    def key(self):
        if not hasattr(self.__class__, '__key__'):
            return getattr(self, '_id', None)

        if isinstance(self.__key__, (list, tuple)):
            if self.__key__[0] == '_id':

                # don`t generate random id twice
                if hasattr(self, '_id'):
                    return self._id

                import bson
                names = self.__key__[1:]
                vals = [
                        unicode(bson.objectid.ObjectId())
                ]
            else:
                names = self.__key__
                vals = []

            vals.extend([
                getattr(self, fieldname)
                for fieldname in names
            ])
            assert None not in vals, "Field must have a value \
if specified in __key__"

            return unicode.join(u"::", vals)

        elif isinstance(self.__key__, basestring):
            return getattr(self, self.__key__)
        elif callable(self.__key__):
            return self.__key__()
        elif hasattr(self.__class__.__key__, 'getter'):
            return self.__key__

        assert False, "Who is Mr. __key__?"

    def save(self):
        _id = getattr(self, '_id', None)
        self._id = mongo.save(self.kind(), self.to_mongo(), _id)
        return self._id

    @classmethod
    def all(cls, **kw):
        return [
                cls(_raw=True, **data)
                for data in
                mongo.find(cls.kind(), **kw)
        ]

    @classmethod
    def kind(cls):
        return cls.__name__.lower()

    @classmethod
    def count(cls):
        return cls.all().count()

    @classmethod
    def get(cls, key=None, **kw):
        if not kw and key:
            kw = {"_id": cls.__key_type(key)}

        data = list(cls.all(**kw)[:1])
        if not data:
            return

        return data[0]

    @classmethod
    def delete(cls, _filter):
        if not isinstance(filter, dict):
            _filter = {"_id": cls.__key_type(_filter)}

        mongo.remove(cls.kind(), _filter)

    def render_form(self, env=None, state='edit', form=None, **kw):
        """
        Form rendering entry point.
        This method can be used to render both forms (model subclasses)
        and form instances (aka objects).
        Returns plain HTML.


        :param env: jinja2 envirement object used for template rendering.
        This param *should* be ommited when calling from jinja2 templates.
        Passed environment should have "form.html" template and widget
        templates availible.

        :param state: controls which state of form should be displayed.
        Common states are "edit" and "table_edit"
        Form states and field states are different concepts.
        Form states are implemented as macros in "form.html" template

        :param form: subform name used when rendering this form/model.
        Subform is a named set (slice) of fields, specified in model.yaml.
        This param should not be passed to forms or objects with subform
        attribute set to value other than None
        When no subforms defined in yaml, no param passed and no "subform"
        attribute set, all fields are rendered

        Default field set can be overriden by specifying "default" subform in
        yaml.
        """
        #print(form, '-', self.subform)
        assert form is None or self.subform is None
        env = env or Environment(loader=PackageLoader('formgear'))
        template = env.get_template('form.html')
        m = getattr(template.module, state, None)

        fields = self.form(form or self.subform or 'default')

        return m(form = fields, **kw)

    _render_form = render_form
    render_form.environmentfunction = True
    render_form = classmethod(render_form)



# -*- coding: utf-8 -*-

import models

def _load(name):
    o = models.MetaModel(name, (models.Model,),
            {'__yaml__': name}
    )

    return o

def load(name):
    ret = models.ModelRegistry.resolve(name, default=False)
    if ret:
        return ret

    return _load(name)



import sys
import logging
import os.path
import re


def find_yaml(yaml_list, dirname, names):
    """
    Callback for `yamls_files` function
    dirname - current directory name, that `yamls_files` walks
    names - list of dirs and files in the `dirname` directory
    yaml_list - accumulator from caller of callback function,
                in this variable we agrigrate obtained files
    """
    for name in names:
        path_to_file = os.path.join(dirname, name)
        if os.path.isfile(path_to_file):
            if ".yaml" in name:
                yaml_list[re.sub("\.yaml", "", name, 1)] = path_to_file


def yamls_files():
    """
    Searching yaml files in sys.path.
    Return the dict like:
        {'yourfilename' : '/absolute/path/to/your/yourfilename.yaml'
         ...
         ...
        }
    """
    yaml_list = {}
    for cur_path in sys.path:
        os.path.walk(cur_path, find_yaml, yaml_list)

    logging.info("obtained yaml files %r" % (yaml_list,))
    return yaml_list


def find_widget_template(widget_path_list, dirname, names):
    if os.path.isdir(dirname) and dirname.endswith("widgets"):
        widget_path_list.append(dirname)


def widgets_path():

    widget_path_list = []
    for cur_path in sys.path:
        os.path.walk(cur_path, find_widget_template, widget_path_list)
    return widget_path_list


def file_resolve(field, name, ypath):
    choicesf = field.get(name, None)

    if choicesf is None:
        return

    if not (isinstance(choicesf, basestring) and \
        choicesf.startswith("@")):
        return

    # try to open csv file
    choices_file = open(os.path.join(os.path.dirname(ypath), choicesf[1:]))
    choices = []
    for line in choices_file:
        line = line.decode("utf-8")
        key, val = line.split("\t", 1)
        choices.append((key, val))

    field[name] = choices

def form_dict(form):
    ret = {}
    for key, val in form:
        container = ret
        while '__' in key:
            ckey, key = key.split('__', 1)
            if ckey.isdigit():
                ckey = int(ckey)
            _container = container.get(ckey, {})
            container[ckey] = _container
            container = _container

        container[key] = val

    return ret




# -*- coding: utf-8 -*-
#
from __future__ import print_function
import os

from jinja2 import Environment, FileSystemLoader
from registry import Registry
from utils import widgets_path


widgetspath = widgets_path()


class NotFoundWidgetException(Exception):
    pass


class WidgetRegistry(Registry):
    NotFound = NotFoundWidgetException


class MetaWidget(type):
    """
    Class for all widgets
    """
    def __new__(cls, name, bases, attrs):
        #print("Forming new widget class:", cls)
        #print("Class bases:", bases)
        #print("Class name:", name)
        #print("Class attrs", attrs)
        meta = attrs.pop('Meta', None)
        abstract = getattr(meta, 'abstract', False)
        registername = attrs.pop('name', name.lower())
        newbornclass = super(MetaWidget, cls).__new__(cls, name, bases, attrs)

        if not abstract:
            newbornclass.load_macros()
            #print("Register widget:", registername)
            WidgetRegistry.register(newbornclass, registername)

        return newbornclass


class Widget(object):
    """
    Виджеты регистрируются в общем реестре и потом могут обратно резолвится
    через специальный реестр.

    Виджеты имеют следующие аттрибуты:

    hidden
       виджет не отображается в форме

    name
      удобное имя для виджета, по умолчанию подставляется имя класса

    template
      шаблон который занимается отображением. Внимание в шаблоне должны быть
      доступный определенный набор макросов для вызова. Для начала наверное что-то
      типа:

        * edit - режим редактирования в стандартной форме
        * view - режим просмотрта
        * table_edit - редактирование в табличной форме (для массового редактирования)
        * table_view - просмотр поля в табличном режиме
        * search_edit - в поисковой форме

    XXX: пока не решен вопрос с AJAX виджетами для которых нужны серверные методы

    """
    __metaclass__ = MetaWidget

    hidden = False
    error_class = 'error'
    css = None
    value = ""
    _type = "text"

    class Meta:
        abstract = True

    def __init__(self, **kw):
        self.__dict__.update(kw)

    def render(self, field, state, env=None, **kw):
        if self.hidden:
            return ''

        macro = getattr(self, '_macro_%s' % state, None)
        assert macro, 'Widget %s have no macro named %r' % (
                self.__class__.__name__, state
        )
        return macro(field=field, widget=self, **kw)

    def itype(self):
        return self._type

    @classmethod
    def load_macros(cls):
        env = Environment(loader=FileSystemLoader(widgetspath))
        tmplt = env.get_template(cls.template+".html")
        mod = tmplt.module
        for macro_name, macro in mod.__dict__.items():
            if macro_name[0] == '_':
                continue

            setattr(cls, '_macro_%s' % macro_name, macro)


# html5 input fields, support status here http://www.w3schools.com/html5/html5_form_input_types.asp
# color
# date
# datetime
# datetime-local
# email
# month
# number
# range
# search
# tel
# time
# url
# week

class DateWidget(Widget):
    """
    DateWidget
    """
    template = 'string'
    alter_names = ('date',)


class StringWidget(Widget):
    """
    InputWidget <input type="text"/>
    """
    _type = 'text'
    alter_names = ('string',)
    template = 'string' # We will try to find text.html template in widgets directory
    value = ""

class PhoneWidget(Widget):
    alter_names = ('tel', 'phone', )
    _type = 'tel'
    template = 'string'

class TextWidget(Widget):
    """
    TextWidget <textarea />
    """
    alter_names = ('text',)
    template = 'text'


class PasswordWidget(StringWidget):
    """
    PasswordWidget <input type="password" />
    """
    alter_names = ('password', 'passw')
    template = 'password'
    _type = 'password'

class BooleanWidget(Widget):
    """" Simple checkbox """
    alter_names = ('boolean', 'bool')
    template = 'boolean'

class EmailWidget(StringWidget):
    """ Email input filed
    """
    alter_names = ('email',)
    template = 'email'
    _type = 'email'

class CheckboxWidget(Widget):
    """
    CheckboxWidget <input type="checkbox" />
    """
    alter_names = ('checkbox',)
    template = 'checkbox'


class PricerangeWidget(Widget):
    alter_names = ('pricerange',)
    template = 'pricerange'


class TimerangeWidget(Widget):
    alter_names = ('timerange',)
    template = 'timerange'


class SelectWidget(Widget):
    """ One choice with multiple list
    """
    alter_names = ('select',)
    template = 'select'

'''class ImageWidget(Widget):
    """
    http://blueimp.github.com/jQuery-File-Upload/
    """
    pass'''

class MarkdownWidget(Widget):
    alter_names = ('markdown',)
    template = 'markdown'



# -*- coding: utf-8 -*-
#

""" Basic validators
"""
import re
from formgear.exceptions import NotFoundValidatorException, InvalidValue
from registry import Registry
import inspect

import sys

PY3 = sys.version_info[0] == 3

if PY3: # pragma: no cover
    string_types = str,
    text_type = str
else:
    string_types = basestring,
    text_type = unicode

class ValidatorRegistry(Registry):
    NotFound = NotFoundValidatorException

class MetaValidator(type):
    """
    Class for all validators
    """
    def __new__(cls, name, bases, attrs):
        registername = attrs.pop('name', name.lower())
        newbornclass = super(MetaValidator, cls).__new__(cls, name, bases, attrs)
        ValidatorRegistry.register(newbornclass, registername)
        return newbornclass

class BaseValidator(object):
    """ All vaidators should be child of this class to add themself to registry
    """
    __metaclass__ = MetaValidator


class Required(BaseValidator):
    """ Validator which tests is value empty."""
    def __init__(self):
        pass

    def __call__(self, node, value):
        if not value:
            raise InvalidValue(node, "is required")

class Regex(BaseValidator):
    def __init__(self, regex, msg=None):
        if isinstance(regex, string_types):
            self.match_object = re.compile(regex)
        else:
            self.match_object = regex
        if msg is None:
            self.msg = "String does not match expected pattern"
        else:
            self.msg = msg

    def __call__(self, node, value):
        if self.match_object.match(value) is None:
            raise InvalidValue(node, self.msg)

class Email(Regex):
    """ Email address validator.
    """
    def __init__(self, msg=None):
        if msg is None:
            msg = _("Invalid email address")
        super(Email, self).__init__(regex='(?i)^[A-Z0-9._%+-]+@[A-Z0-9.-]+\.[A-Z]{2,4}$', msg=msg)

class Length(BaseValidator):
    """ Validator which succeeds if the value passed to it has a
    length between a minimum and maximum.  The value is most often a
    string."""
    min_err = 'Shorter than minimum length %s'
    max_err = 'Longer than maximum length %s'

    def __init__(self, min=None, max=None):
        self.min = min
        self.max = max

    def __call__(self, node, value):
        if self.min is not None:
            if len(value) < self.min:
                raise InvalidValue(node, self.min_err % str(self.min))

        if self.max is not None:
            if len(value) > self.max:
                raise InvalidValue(node, self.max_err % str(self.max))

class Max(BaseValidator):
    """ Validate if value not great than `value`
    """

    text_err = 'Current value - %d is great than max %d'

    def __init__(self, value=None):
        self.value = value

    def __call__(self, node, value):
        if self.value is not None:
            if self.value < value:
                raise InvalidValue(node, self.text_err % (self.value, value,))

class Min(BaseValidator):
    """ Validate if value not less than `value`
    """

    text_err = 'Current value - %d is less than min %d'

    def __init__(self, value=None):
        self.value = value

    def __call__(self, node, value):
        if self.value is not None:
            if self.value > value:
                raise InvalidValue(node, self.text_err % (self.value, value,))


def lookup(lvs={}):
    return [ValidatorRegistry.resolve(validator)(**lvs[validator]) for validator in lvs]



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from distutils.core import setup

setup(name='formgear',
      version='0.3.2',
      description='Form, mongodbengine objects',
      long_description = open("README.rst").read(),
      author='Mikhail Kashkin',
      author_email='mkashkin@gmail.com',
      url='https://github.com/xen/formgear',
      # more examples here http://docs.python.org/distutils/examples.html#pure-python-distribution-by-package
      packages=['formgear', ],
      license = "BSD",
      install_requires=[
          'jinja2',
          'pyyaml'
      ],
      package_data = {
        'formgear': [
            'templates/*.html',
            'templates/widgets/*.html',
        ]
    }
     )
